/*
 *  texture.cpp
 *  GL
 *
 *  Created by NYU User on 11/30/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */
#include <iostream>
#include <cassert>	//for assert

#include <GLTools.h>			//OpenGL Toolkit
#include <GLShaderManager.h>	//Shader Manager Class
#include <GLFrustum.h>
#include <GLMatrixStack.h>
#include <GLGeometryTransform.h>

GLShaderManager shaderManager;
GLFrustum frustum;
GLMatrixStack projection;
GLMatrixStack modelView;
GLGeometryTransform geometryTransform;
GLuint textureID;

GLTriangleBatch triangle;
//GLTriangleBatch sphere;

//Angles in radians changed by the touchesMoved:withEvent: method of EAGLView.
GLfloat theta = 0.0f;	//longitude
GLfloat phi = 0.0f;		//latitude

void printError(const char *s)
{
	while (const GLenum error = glGetError()) {
		switch (error) {
		case GL_INVALID_ENUM:
			std::cerr << s << ": GL_INVALID_ENUM\n";
			break;
		case GL_INVALID_VALUE:
			std::cerr << s << ": GL_INVALID_VALUE\n";
			break;
		case GL_INVALID_OPERATION:
			std::cerr << s << ": GL_INVALID_OPERATION\n";
			break;
		case GL_INVALID_FRAMEBUFFER_OPERATION:
			std::cerr << s << ": GL_INVALID_FRAMEBUFFER_OPERATION\n";
			break;
		case GL_OUT_OF_MEMORY:
			std::cerr << s << ": GL_OUT_OF_MEMORY\n";
			break;
		case GL_NO_ERROR:
			break;
		default:
			std::cerr << "error == " << error << "\n";
		}
	}
}
			
// Load a TGA as a 2D Texture.  Completely initialize the state.
bool LoadTGATexture(const char *szFileName, GLenum minFilter, GLenum magFilter, GLenum wrapMode)
{
	int nWidth, nHeight, nComponents;
	GLenum eFormat;
	
	// Read the texture bits.
	if (GLbyte *const pBits =
		gltReadTGABits(szFileName, &nWidth, &nHeight, &nComponents, &eFormat)) {

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrapMode);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapMode);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minFilter);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magFilter);

		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glTexImage2D(
			GL_TEXTURE_2D,
			0,	//mapmap level
			nComponents,
			nWidth, nHeight,
			0,	//border
			eFormat,
			GL_UNSIGNED_BYTE,
			pBits
		);
	
		free(pBits);
    
		switch (minFilter) {
			case GL_LINEAR_MIPMAP_LINEAR: 
			case GL_LINEAR_MIPMAP_NEAREST:
			case GL_NEAREST_MIPMAP_LINEAR:
			case GL_NEAREST_MIPMAP_NEAREST:
				glGenerateMipmap(GL_TEXTURE_2D);
				break;
		}
		return true;
	}

	return false;
}

void SetupRC()
{
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);	//blue background
	shaderManager.InitializeStockShaders();
	glEnable(GL_DEPTH_TEST);
	glGenTextures(1, &textureID);
    glBindTexture(GL_TEXTURE_2D, textureID);
	
    if (!LoadTGATexture("WorldTile.tga", GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE)) {
		std::cerr << "LoadTGATexture failed.\n";
		return;
	}

	//gltMakeSphere(sphere, 1.0f, 40, 20);
	
	const GLfloat r = 1.0f;
	const size_t nVertices = 3;
	triangle.BeginMesh(nVertices);
	
	M3DVector3f vertices[] = {
		{r * cosf(m3dDegToRad( 90.0f)), r * sinf(m3dDegToRad( 90.0f)), 0.0f},	//top
		{r * cosf(m3dDegToRad(210.0f)), r * sinf(m3dDegToRad(210.0f)), 0.0f},	//lower left
		{r * cosf(m3dDegToRad(330.0f)), r * sinf(m3dDegToRad(330.0f)), 0.0f}	//lowre right
	};
	
	M3DVector3f normals[] = {
		{0.0f, 0.0f, 1.0f},
		{0.0f, 0.0f, 1.0f},
		{0.0f, 0.0f, 1.0f}
	};
	
	M3DVector2f textureCoordinates[] = {
		{0.5f, 1.0f},
		{0.0f, 0.0f},
		{1.0f, 0.0f}
	};
	
	triangle.AddTriangle(vertices, normals, textureCoordinates);
	triangle.End();
}

void reshape(GLsizei width, GLsizei height)
{
	//Fill the entire iPhone screen.
	glViewport(0, 0, width, height);
	
	//Make closer objects look bigger, farther smaller.
	
	frustum.SetPerspective(
		35.0f,		//vertical field of view in degrees
		static_cast<GLfloat>(width) / height,
		1.0f,		//nearest visible distance
		10.0f		//farthest
	);
	
	projection.LoadMatrix(frustum.GetProjectionMatrix());
	geometryTransform.SetMatrixStacks(modelView, projection);
}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	modelView.PushMatrix();
	assert(modelView.GetLastError() == GLT_STACK_NOERROR);
	
	//A positive value of z would put the triangle *behind* us
	//and we wouldn't see it.
	modelView.Translate(0.0f, 0.0f, -7.0f);
	
	modelView.Rotate(m3dRadToDeg(phi), 0.0f, 1.0f, 0.0f);
	modelView.Rotate(m3dRadToDeg(theta), 1.0f, 0.0f, 0.0f);
		
	static const M3DVector4f lightPosition = {0.0f, 10.0f, 5.0f, 1.0f};
	static const GLfloat white[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glBindTexture(GL_TEXTURE_2D, textureID);
	
	//std::cerr << "textureID == " << textureID << "\n";
	shaderManager.UseStockShader(
		GLT_SHADER_TEXTURE_POINT_LIGHT_DIFF,
		geometryTransform.GetModelViewMatrix(),
		geometryTransform.GetProjectionMatrix(),
		lightPosition,
		white,
		0
	);
	
	triangle.Draw();
	
/*	
	glBindTexture(GL_TEXTURE_2D, textureID);
	shaderManager.UseStockShader(
		GLT_SHADER_TEXTURE_POINT_LIGHT_DIFF,
		geometryTransform.GetModelViewMatrix(),
		geometryTransform.GetProjectionMatrix(),
		lightPosition,
		white,
		0
	);
	sphere.Draw();
*/	
 
	modelView.PopMatrix();
	assert(modelView.GetLastError() == GLT_STACK_NOERROR);
}

void ShutdownRC()
{
    glDeleteTextures(1, &textureID);
}
